@@ -75,13 +75,10 @@ static void     split_mimetype( const gchar *mimetype, gchar **group, gchar **su
 static gboolean is_candidate_for_basenames( const NAIContext *object, guint target, GList *files );
 static gboolean is_candidate_for_selection_count( const NAIContext *object, guint target, GList *files );
 static gboolean is_candidate_for_schemes( const NAIContext *object, guint target, GList *files );
-#if 0
-static gboolean validate_schemes( GSList *object_schemes, NASelectedInfo *iter );
-#endif
+static gboolean is_compatible_scheme( const gchar *pattern, const gchar *scheme );
 static gboolean is_candidate_for_folders( const NAIContext *object, guint target, GList *files );
 static gboolean is_candidate_for_capabilities( const NAIContext *object, guint target, GList *files );
 
-static void     count_compatible_patterns( gboolean positive, gboolean match, guint *count_compatible, gboolean *ok );
 static gboolean is_valid_basenames( const NAIContext *object );
 static gboolean is_valid_mimetypes( const NAIContext *object );
 static gboolean is_valid_isfiledir( const NAIContext *object );
@@ -171,49 +168,49 @@ interface_base_finalize( NAIContextInterface *klass )
 
 /**
  * na_icontext_is_candidate:
- * @object: a #NAIContext to be checked.
+ * @context: a #NAIContext to be checked.
  * @target: the current target.
- * @files: the currently selected items, as a #GList of #NASelectedInfo items.
+ * @selection: the currently selected items, as a #GList of #NASelectedInfo items.
  *
  * Determines if the given object may be candidate to be displayed in
  * the Nautilus context menu, depending of the list of currently selected
  * items.
  *
  * This function is called by nautilus-actions::build_nautilus_menus()
- * for each item found in NAPivot items list, and, when this an action,
+ * for each item found in #NAPivot items list, and, when this an action,
  * for each profile of this action.
  *
- * Returns: %TRUE if this object succeeds to all tests and is so a
+ * Returns: %TRUE if this @context succeeds to all tests and is so a
  * valid candidate to be displayed in Nautilus context menu, %FALSE
  * else.
  */
 gboolean
-na_icontext_is_candidate( const NAIContext *object, guint target, GList *files )
+na_icontext_is_candidate( const NAIContext *context, guint target, GList *selection )
 {
 	static const gchar *thisfn = "na_icontext_is_candidate";
 	gboolean is_candidate;
 
-	g_return_val_if_fail( NA_IS_ICONTEXT( object ), FALSE );
+	g_return_val_if_fail( NA_IS_ICONTEXT( context ), FALSE );
 
-	g_debug( "%s: object=%p (%s), target=%d, files=%p (count=%d)",
-			thisfn, ( void * ) object, G_OBJECT_TYPE_NAME( object ), target, (void * ) files, g_list_length( files ));
+	g_debug( "%s: object=%p (%s), target=%d, selection=%p (count=%d)",
+			thisfn, ( void * ) context, G_OBJECT_TYPE_NAME( context ), target, (void * ) selection, g_list_length( selection ));
 
-	is_candidate = v_is_candidate( NA_ICONTEXT( object ), target, files );
+	is_candidate = v_is_candidate( NA_ICONTEXT( context ), target, selection );
 
 	if( is_candidate ){
 		is_candidate =
-				is_candidate_for_target( object, target, files ) &&
-				is_candidate_for_show_in( object, target, files ) &&
-				is_candidate_for_try_exec( object, target, files ) &&
-				is_candidate_for_show_if_registered( object, target, files ) &&
-				is_candidate_for_show_if_true( object, target, files ) &&
-				is_candidate_for_show_if_running( object, target, files ) &&
-				is_candidate_for_mimetypes( object, target, files ) &&
-				is_candidate_for_basenames( object, target, files ) &&
-				is_candidate_for_selection_count( object, target, files ) &&
-				is_candidate_for_schemes( object, target, files ) &&
-				is_candidate_for_folders( object, target, files ) &&
-				is_candidate_for_capabilities( object, target, files );
+				is_candidate_for_target( context, target, selection ) &&
+				is_candidate_for_show_in( context, target, selection ) &&
+				is_candidate_for_try_exec( context, target, selection ) &&
+				is_candidate_for_show_if_registered( context, target, selection ) &&
+				is_candidate_for_show_if_true( context, target, selection ) &&
+				is_candidate_for_show_if_running( context, target, selection ) &&
+				is_candidate_for_mimetypes( context, target, selection ) &&
+				is_candidate_for_basenames( context, target, selection ) &&
+				is_candidate_for_selection_count( context, target, selection ) &&
+				is_candidate_for_schemes( context, target, selection ) &&
+				is_candidate_for_folders( context, target, selection ) &&
+				is_candidate_for_capabilities( context, target, selection );
 	}
 
 	return( is_candidate );
@@ -221,29 +218,29 @@ na_icontext_is_candidate( const NAIContext *object, guint target, GList *files )
 
 /**
  * na_icontext_is_valid:
- * @profile: the #NAObjectProfile to be checked.
+ * @context: the #NAObjectProfile to be checked.
  *
- * Returns: %TRUE if this profile is valid, %FALSE else.
+ * Returns: %TRUE if this @context is valid, %FALSE else.
  *
- * This function is part of NAIDuplicable::check_status() and is called
- * by NAIDuplicable objects which also implement NAIContext
+ * This function is part of #NAIDuplicable::check_status() and is called
+ * by #NAIDuplicable objects which also implement #NAIContext
  * interface. It so doesn't make sense of asking the object for its
  * validity status as it has already been checked before calling the
  * function.
  */
 gboolean
-na_icontext_is_valid( const NAIContext *object )
+na_icontext_is_valid( const NAIContext *context )
 {
 	gboolean is_valid;
 
-	g_return_val_if_fail( NA_IS_ICONTEXT( object ), FALSE );
+	g_return_val_if_fail( NA_IS_ICONTEXT( context ), FALSE );
 
 	is_valid =
-		is_valid_basenames( object ) &&
-		is_valid_mimetypes( object ) &&
-		is_valid_isfiledir( object ) &&
-		is_valid_schemes( object ) &&
-		is_valid_folders( object );
+		is_valid_basenames( context ) &&
+		is_valid_mimetypes( context ) &&
+		is_valid_isfiledir( context ) &&
+		is_valid_schemes( context ) &&
+		is_valid_folders( context );
 
 	return( is_valid );
 }
@@ -252,18 +249,18 @@ na_icontext_is_valid( const NAIContext *object )
  * na_icontext_is_all_mimetypes:
  * @context: the #NAIContext object to be checked.
  *
- * Returns: %TRUE if this context is valid for all mimetypes, %FALSE else.
+ * Returns: %TRUE if this @context is valid for all mimetypes, %FALSE else.
  */
 gboolean
-na_icontext_is_all_mimetypes( const NAIContext *object )
+na_icontext_is_all_mimetypes( const NAIContext *context )
 {
 	gboolean is_all;
 	GSList *mimetypes, *im;
 
-	g_return_val_if_fail( NA_IS_ICONTEXT( object ), FALSE );
+	g_return_val_if_fail( NA_IS_ICONTEXT( context ), FALSE );
 
 	is_all = TRUE;
-	mimetypes = na_object_get_mimetypes( object );
+	mimetypes = na_object_get_mimetypes( context );
 
 	for( im = mimetypes ; im ; im = im->next ){
 		if( !im->data || !strlen( im->data )){
@@ -298,11 +295,11 @@ na_icontext_read_done( NAIContext *context )
 
 /**
  * na_icontext_set_scheme:
- * @profile: the #NAIContext to be updated.
+ * @context: the #NAIContext to be updated.
  * @scheme: name of the scheme.
- * @selected: whether this scheme is candidate to this context.
+ * @selected: whether this scheme is candidate to this @context.
  *
- * Sets the status of a scheme relative to this context.
+ * Sets the status of a @scheme relative to this @context.
  */
 void
 na_icontext_set_scheme( NAIContext *context, const gchar *scheme, gboolean selected )
@@ -319,11 +316,11 @@ na_icontext_set_scheme( NAIContext *context, const gchar *scheme, gboolean selec
 
 /**
  * na_icontext_set_only_desktop:
- * @profile: the #NAIContext to be updated.
+ * @context: the #NAIContext to be updated.
  * @desktop: name of the desktop environment.
- * @selected: whether this desktop is candidate to this context.
+ * @selected: whether this @desktop is candidate to this @context.
  *
- * Sets the status of the desktop relative to this context for the OnlyShowIn list.
+ * Sets the status of the @desktop relative to this @context for the OnlyShowIn list.
  */
 void
 na_icontext_set_only_desktop( NAIContext *context, const gchar *desktop, gboolean selected )
@@ -340,11 +337,11 @@ na_icontext_set_only_desktop( NAIContext *context, const gchar *desktop, gboolea
 
 /**
  * na_icontext_set_only_desktop:
- * @profile: the #NAIContext to be updated.
+ * @context: the #NAIContext to be updated.
  * @desktop: name of the desktop environment.
- * @selected: whether this desktop is candidate to this context.
+ * @selected: whether this @desktop is candidate to this @context.
  *
- * Sets the status of the desktop relative to this context for the NotShowIn list.
+ * Sets the status of the @desktop relative to this @context for the NotShowIn list.
  */
 void
 na_icontext_set_not_desktop( NAIContext *context, const gchar *desktop, gboolean selected )
@@ -361,35 +358,35 @@ na_icontext_set_not_desktop( NAIContext *context, const gchar *desktop, gboolean
 
 /**
  * na_icontext_replace_folder:
- * @profile: the #NAIContext to be updated.
+ * @context: the #NAIContext to be updated.
  * @old: the old uri.
  * @new: the new uri.
  *
  * Replaces the @old URI by the @new one.
  */
 void
-na_icontext_replace_folder( NAIContext *profile, const gchar *old, const gchar *new )
+na_icontext_replace_folder( NAIContext *context, const gchar *old, const gchar *new )
 {
 	GSList *folders;
 
-	g_return_if_fail( NA_IS_ICONTEXT( profile ));
+	g_return_if_fail( NA_IS_ICONTEXT( context ));
 
-	folders = na_object_get_folders( profile );
+	folders = na_object_get_folders( context );
 	folders = na_core_utils_slist_remove_utf8( folders, old );
 	folders = g_slist_append( folders, ( gpointer ) g_strdup( new ));
-	na_object_set_folders( profile, folders );
+	na_object_set_folders( context, folders );
 	na_core_utils_slist_free( folders );
 }
 
 static gboolean
-v_is_candidate( NAIContext *object, guint target, GList *selection )
+v_is_candidate( NAIContext *context, guint target, GList *selection )
 {
 	gboolean is_candidate;
 
 	is_candidate = TRUE;
 
-	if( NA_ICONTEXT_GET_INTERFACE( object )->is_candidate ){
-		is_candidate = NA_ICONTEXT_GET_INTERFACE( object )->is_candidate( object, target, selection );
+	if( NA_ICONTEXT_GET_INTERFACE( context )->is_candidate ){
+		is_candidate = NA_ICONTEXT_GET_INTERFACE( context )->is_candidate( context, target, selection );
 	}
 
 	return( is_candidate );
@@ -624,6 +621,14 @@ is_candidate_for_show_if_running( const NAIContext *object, guint target, GList
  * file mimetype must be compatible with at least one positive assertion
  * (they are ORed), while not being of any negative assertions (they are
  * ANDed)
+ *
+ * here, for each mimetype of the selection, do
+ * . match is FALSE while this mimetype has not matched a positive assertion
+ * . nomatch is FALSE while this mimetype has not matched a negative assertion
+ * we are going to check each mimetype filter
+ *  while we have not found a match among positive conditions (i.e. while !match)
+ *  we have to check all negative conditions to verify that the current
+ *  examined mimetype never match these
  */
 static gboolean
 is_candidate_for_mimetypes( const NAIContext *object, guint target, GList *files )
@@ -631,39 +636,42 @@ is_candidate_for_mimetypes( const NAIContext *object, guint target, GList *files
 	static const gchar *thisfn = "na_icontext_is_candidate_for_mimetypes";
 	gboolean ok = TRUE;
 	gboolean all = na_object_get_all_mimetypes( object );
-	GList *it;
 
 	g_debug( "%s: all=%s", thisfn, all ? "True":"False" );
 
 	if( !all ){
 		GSList *mimetypes = na_object_get_mimetypes( object );
 		GSList *im;
-		gboolean positive;
-		guint count_positive = 0;
-		guint count_compatible = 0;
+		GList *it;
 
-		for( im = mimetypes ; im && ok ; im = im->next ){
-			const gchar *imtype = ( const gchar * ) im->data;
-			positive = is_positive_assertion( imtype );
-			if( positive ){
-				count_positive += 1;
-			}
-			gchar *imgroup, *imsubgroup;
-			split_mimetype( positive ? imtype : imtype+1, &imgroup, &imsubgroup );
+		for( it = files ; it && ok ; it = it->next ){
+			gchar *ftype, *fgroup, *fsubgroup;
+			gboolean match, positive;
 
-			for( it = files ; it && ok ; it = it->next ){
-				gchar *ftype = na_selected_info_get_mime_type( NA_SELECTED_INFO( it->data ));
-				gboolean type_of = is_mimetype_of( ftype, imgroup, imsubgroup );
-				count_compatible_patterns( positive, type_of, &count_compatible, &ok );
-				g_free( ftype );
+			ftype = na_selected_info_get_mime_type( NA_SELECTED_INFO( it->data ));
+			split_mimetype( ftype, &fgroup, &fsubgroup );
+			match = FALSE;
+
+			for( im = mimetypes ; im && ok ; im = im->next ){
+				const gchar *imtype = ( const gchar * ) im->data;
+				positive = is_positive_assertion( imtype );
+
+				if( !positive || !match ){
+					if( is_mimetype_of( positive ? imtype : imtype+1, fgroup, fsubgroup )){
+						if( positive ){
+							match = TRUE;
+						} else {
+							ok = FALSE;
+						}
+					}
+				}
 			}
 
-			g_free( imgroup );
-			g_free( imsubgroup );
-		}
+			ok &= match;
 
-		if( count_positive > 0 && count_compatible == 0 ){
-			ok = FALSE;
+			g_free( fsubgroup );
+			g_free( fgroup );
+			g_free( ftype );
 		}
 
 		if( !ok ){
@@ -679,31 +687,35 @@ is_candidate_for_mimetypes( const NAIContext *object, guint target, GList *files
 }
 
 /*
- * does the file have a mimetype which is 'a sort of' specified one ?
+ * does the file fgroup/fsubgrop have a mimetype which is 'a sort of'
+ *  mimetype specified one ?
  * for example, "image/jpeg" is clearly a sort of "image/ *"
  * but how to check to see if "msword/xml" is a sort of "application/xml" ??
  */
 static gboolean
-is_mimetype_of( const gchar *file_type, const gchar *group, const gchar *subgroup )
+is_mimetype_of( const gchar *mimetype, const gchar *fgroup, const gchar *fsubgroup )
 {
 	gboolean is_type_of;
-	gchar *fgroup, *fsubgroup;
+	gchar *mgroup, *msubgroup;
 
-	split_mimetype( file_type, &fgroup, &fsubgroup );
+	split_mimetype( mimetype, &mgroup, &msubgroup );
 
-	is_type_of = ( strcmp( fgroup, group ) == 0 );
+	is_type_of = ( strcmp( fgroup, mgroup ) == 0 );
 	if( is_type_of ){
-		if( strcmp( subgroup, "*" )){
-			is_type_of = ( strcmp( fsubgroup, subgroup ) == 0 );
+		if( strcmp( msubgroup, "*" ) != 0 ){
+			is_type_of = ( strcmp( fsubgroup, msubgroup ) == 0 );
 		}
 	}
 
-	g_free( fgroup );
-	g_free( fsubgroup );
+	g_free( mgroup );
+	g_free( msubgroup );
 
 	return( is_type_of );
 }
 
+/*
+ * split the given mimetype to its group and subgroup components
+ */
 static void
 split_mimetype( const gchar *mimetype, gchar **group, gchar **subgroup )
 {
@@ -721,49 +733,47 @@ is_candidate_for_basenames( const NAIContext *object, guint target, GList *files
 	static const gchar *thisfn = "na_icontext_is_candidate_for_basenames";
 	gboolean ok = TRUE;
 	GSList *basenames = na_object_get_basenames( object );
-	GSList *ib;
-	gboolean positive;
-	guint count_positive = 0;
-	guint count_compatible = 0;
-	gchar *pattern, *bname, *bname_utf8;
-	GList *it;
-	GPatternSpec *pattern_spec;
-	gboolean match;
 
 	if( basenames ){
 		if( strcmp( basenames->data, "*" ) != 0 || g_slist_length( basenames ) > 1 ){
 			gboolean matchcase = na_object_get_matchcase( object );
+			GSList *ib;
+			GList *it;
 
-			for( ib = basenames ; ib && ok ; ib = ib->next ){
-				pattern = matchcase ?
+			for( it = files ; it && ok ; it = it->next ){
+				gchar *pattern, *bname, *bname_utf8;
+				gboolean match, positive;
+
+				bname = na_selected_info_get_basename( NA_SELECTED_INFO( it->data ));
+				bname_utf8 = g_filename_to_utf8( bname, -1, NULL, NULL, NULL );
+				match = FALSE;
+
+				for( ib = basenames ; ib && ok ; ib = ib->next ){
+					pattern = matchcase ?
 						g_strdup(( gchar * ) ib->data ) :
 						g_ascii_strdown(( gchar * ) ib->data, strlen(( gchar * ) ib->data ));
-				positive = is_positive_assertion( pattern );
-				if( positive ){
-					count_positive += 1;
+					positive = is_positive_assertion( pattern );
+
+					if( !positive || !match ){
+						if( g_pattern_match_simple( positive ? pattern : pattern+1, bname_utf8 )){
+							if( positive ){
+								match = TRUE;
+							} else {
+								ok = FALSE;
+							}
+						}
+					}
+
+					g_free( pattern );
 				}
 
-				pattern_spec = g_pattern_spec_new( positive ? pattern : pattern+1 );
+				ok &= match;
 
-				for( it = files ; it && ok ; it = it->next ){
-					bname = na_selected_info_get_basename( NA_SELECTED_INFO( it->data ));
-					bname_utf8 = g_filename_to_utf8( bname, -1, NULL, NULL, NULL );
-					match = g_pattern_match_string( pattern_spec, bname_utf8 );
-					g_free( bname_utf8 );
-					g_free( bname );
-					g_debug( "%s: pattern=%s, positive=%s, match=%s", thisfn, pattern, positive ? "True":"False", match ? "True":"False" );
-					count_compatible_patterns( positive, match, &count_compatible, &ok );
-				}
-
-				g_pattern_spec_free( pattern_spec );
-				g_free( pattern );
+				g_free( bname_utf8 );
+				g_free( bname );
 			}
 		}
 
-		if( count_positive > 0 && count_compatible == 0 ){
-			ok = FALSE;
-		}
-
 		if( !ok ){
 			gchar *basenames_str = na_core_utils_slist_to_text( basenames );
 			g_debug( "%s: object is not candidate because Basenames=%s", thisfn, basenames_str );
@@ -814,45 +824,61 @@ is_candidate_for_selection_count( const NAIContext *object, guint target, GList
 	return( ok );
 }
 
+/*
+ * it is likely that all selected items have the same scheme, because they
+ * are all in the same location and the scheme mainly depends on location
+ * so we have here a great possible optimization by only testing the
+ * first selected item.
+ * note that this optimization may be wrong, for example when ran from the
+ * command-line with a random set of pseudo-selected items
+ * so we take care of only checking _distincts_ schemes of provided selection
+ * against schemes conditions.
+ */
 static gboolean
 is_candidate_for_schemes( const NAIContext *object, guint target, GList *files )
 {
 	static const gchar *thisfn = "na_icontext_is_candidate_for_schemes";
 	gboolean ok = TRUE;
 	GSList *schemes = na_object_get_schemes( object );
-	GSList *is;
-	gboolean positive;
-	guint count_positive = 0;
-	guint count_compatible = 0;
-	gchar *pattern, *scheme;
-	GList *it;
-	gboolean match;
 
 	if( schemes ){
-		for( is = schemes ; is && ok ; is = is->next ){
-			pattern = ( gchar * ) is->data;
-			positive = is_positive_assertion( pattern );
-			if( positive ){
-				count_positive += 1;
-			} else {
-				pattern += 1;
-			}
+		if( strcmp( schemes->data, "*" ) != 0 || g_slist_length( schemes ) > 1 ){
+			GSList *distincts = NULL;
+			GList *it;
 
-			if( strcmp( pattern, "*" ) == 0 ){
-				count_compatible_patterns( positive, TRUE, &count_compatible, &ok );
+			for( it = files ; it && ok ; it = it->next ){
+				gchar *scheme = na_selected_info_get_uri_scheme( NA_SELECTED_INFO( files->data ));
 
-			} else {
-				for( it = files ; it && ok ; it = it->next ){
-					scheme = na_selected_info_get_uri_scheme( NA_SELECTED_INFO( it->data ));
-					match = ( strcmp( pattern, scheme ) == 0 );
-					g_free( scheme );
-					count_compatible_patterns( positive, match, &count_compatible, &ok );
+				if( na_core_utils_slist_count( distincts, scheme ) == 0 ){
+					GSList *is;
+					gchar *scheme, *pattern;
+					gboolean match, positive;
+
+					match = FALSE;
+					distincts = g_slist_prepend( distincts, g_strdup( scheme ));
+
+					for( is = schemes ; is && ok ; is = is->next ){
+						pattern = ( gchar * ) is->data;
+						positive = is_positive_assertion( pattern );
+
+						if( !positive || !match ){
+							if( is_compatible_scheme( positive ? pattern : pattern+1, scheme )){
+								if( positive ){
+									match = TRUE;
+								} else {
+									ok = FALSE;
+								}
+							}
+						}
+					}
+
+					ok &= match;
 				}
-			}
-		}
 
-		if( count_positive > 0 && count_compatible == 0 ){
-			ok = FALSE;
+				g_free( scheme );
+			}
+
+			na_core_utils_slist_free( distincts );
 		}
 
 		if( !ok ){
@@ -867,62 +893,78 @@ is_candidate_for_schemes( const NAIContext *object, guint target, GList *files )
 	return( ok );
 }
 
+static gboolean
+is_compatible_scheme( const gchar *pattern, const gchar *scheme )
+{
+	gboolean compatible;
+
+	compatible = FALSE;
+
+	if( strcmp( pattern, "*" )){
+		compatible = TRUE;
+	} else {
+		compatible = ( strcmp( pattern, scheme ) == 0 );
+	}
+
+	return( compatible );
+}
+
+/*
+ * assumuing here the same sort of optimization than for schemes
+ * i.e. we assume that all selected items are must probably located
+ * in the same dirname
+ * so we take care of only checking _distinct_ dirnames against folder
+ * conditions
+ */
 static gboolean
 is_candidate_for_folders( const NAIContext *object, guint target, GList *files )
 {
 	static const gchar *thisfn = "na_icontext_is_candidate_for_folders";
 	gboolean ok = TRUE;
 	GSList *folders = na_object_get_folders( object );
-	GSList *id;
-	const gchar *pattern;
-	GList *it;
-	gchar *dirname, *dirname_utf8;
-	gboolean positive, match;
-	guint count_positive = 0;
-	guint count_compatible = 0;
-	GPatternSpec *pattern_spec;
 
 	if( folders ){
 		if( strcmp( folders->data, "/" ) != 0 || g_slist_length( folders ) > 1 ){
-			for( id = folders ; id && ok ; id = id->next ){
-				pattern = ( const gchar * ) id->data;
-				positive = is_positive_assertion( pattern );
-				if( positive ){
-					count_positive += 1;
-				} else {
-					pattern += 1;
-				}
+			GSList *distincts = NULL;
+			GList *it;
 
-				pattern_spec = NULL;
-				if( g_strstr_len( pattern, -1, "*" ) != NULL ){
-					pattern_spec = g_pattern_spec_new( pattern );
-				}
+			for( it = files ; it && ok ; it = it->next ){
+				gchar *dirname = na_selected_info_get_dirname( NA_SELECTED_INFO( files->data ));
 
-				for( it = files ; it && ok ; it = it->next ){
-					dirname = na_selected_info_get_dirname( NA_SELECTED_INFO( it->data ));
+				if( na_core_utils_slist_count( distincts, dirname ) == 0 ){
+					GSList *id;
+					gchar *dirname_utf8;
+					const gchar *pattern;
+					gboolean match, positive;
 
-					if( pattern_spec ){
-						dirname_utf8 = g_filename_to_utf8( dirname, -1, NULL, NULL, NULL );
-						match = g_pattern_match_string( pattern_spec, dirname_utf8 );
-						g_free( dirname_utf8 );
+					distincts = g_slist_prepend( distincts, g_strdup( dirname ));
+					dirname_utf8 = g_filename_to_utf8( dirname, -1, NULL, NULL, NULL );
+					match = FALSE;
 
-					} else {
-						match = g_str_has_prefix( dirname, pattern );
+					for( id = folders ; id && ok ; id = id->next ){
+						pattern = ( const gchar * ) id->data;
+						positive = is_positive_assertion( pattern );
+
+						if( !positive || !match ){
+							if( g_pattern_match_simple( positive ? pattern : pattern+1, dirname_utf8 )){
+								if( positive ){
+									match = TRUE;
+								} else {
+									ok = FALSE;
+								}
+							}
+						}
 					}
 
-					g_free( dirname );
-					g_debug( "%s: pattern=%s, positive=%s, match=%s", thisfn, pattern, positive ? "True":"False", match ? "True":"False" );
-					count_compatible_patterns( positive, match, &count_compatible, &ok );
+					ok &= match;
+
+					g_free( dirname_utf8 );
 				}
 
-				if( pattern_spec ){
-					g_pattern_spec_free( pattern_spec );
-				}
+				g_free( dirname );
 			}
-		}
 
-		if( count_positive > 0 && count_compatible == 0 ){
-			ok = FALSE;
+			na_core_utils_slist_free( distincts );
 		}
 
 		if( !ok ){
@@ -943,55 +985,42 @@ is_candidate_for_capabilities( const NAIContext *object, guint target, GList *fi
 	static const gchar *thisfn = "na_icontext_is_candidate_for_capabilities";
 	gboolean ok = TRUE;
 	GSList *capabilities = na_object_get_capabilities( object );
-	GSList *ic;
-	const gchar *cap;
-	GList *it;
-	gboolean positive, match;
-	guint count_positive = 0;
-	guint count_compatible = 0;
 
 	if( capabilities ){
-		for( ic = capabilities ; ic && ok ; ic = ic->next ){
-			cap = ( const gchar * ) ic->data;
-			positive = is_positive_assertion( cap );
-			if( positive ){
-				count_positive += 1;
-			} else {
-				cap += 1;
-			}
+		GSList *ic;
+		GList *it;
+		const gchar *cap;
+		gboolean match, positive;
 
-			for( it = files ; it && ok ; it = it->next ){
-				if( !strcmp( cap, "Owner" )){
+		for( it = files ; it && ok ; it = it->next ){
+			for( ic = capabilities ; ic && ok ; ic = ic->next ){
+				cap = ( const gchar * ) ic->data;
+				positive = is_positive_assertion( cap );
+				match = FALSE;
+
+				if( !strcmp( positive ? cap : cap+1, "Owner" )){
 					match = na_selected_info_is_owner( NA_SELECTED_INFO( it->data ), getlogin());
-					count_compatible_patterns( positive, match, &count_compatible, &ok );
 
-				} else if( !strcmp( cap, "Readable" )){
+				} else if( !strcmp( positive ? cap : cap+1, "Readable" )){
 					match = na_selected_info_is_readable( NA_SELECTED_INFO( it->data ));
-					count_compatible_patterns( positive, match, &count_compatible, &ok );
 
-				} else if( !strcmp( cap, "Writable" )){
+				} else if( !strcmp( positive ? cap : cap+1, "Writable" )){
 					match = na_selected_info_is_writable( NA_SELECTED_INFO( it->data ));
-					count_compatible_patterns( positive, match, &count_compatible, &ok );
 
-				} else if( !strcmp( cap, "Executable" )){
+				} else if( !strcmp( positive ? cap : cap+1, "Executable" )){
 					match = na_selected_info_is_executable( NA_SELECTED_INFO( it->data ));
-					count_compatible_patterns( positive, match, &count_compatible, &ok );
 
-				} else if( !strcmp( cap, "Local" )){
+				} else if( !strcmp( positive ? cap : cap+1, "Local" )){
 					match = na_selected_info_is_local( NA_SELECTED_INFO( it->data ));
-					count_compatible_patterns( positive, match, &count_compatible, &ok );
 
 				} else {
 					g_warning( "%s: unknown capability %s", thisfn, cap );
 				}
+
+				ok &= (( positive && match ) || ( !positive && !match ));
 			}
 		}
 
-		if( count_positive > 0 && count_compatible == 0 ){
-			ok = FALSE;
-		}
-
-
 		if( !ok ){
 			gchar *capabilities_str = na_core_utils_slist_to_text( capabilities );
 			g_debug( "%s: object is not candidate because Capabilities=%s", thisfn, capabilities_str );
@@ -1004,18 +1033,6 @@ is_candidate_for_capabilities( const NAIContext *object, guint target, GList *fi
 	return( ok );
 }
 
-static void
-count_compatible_patterns( gboolean positive, gboolean match, guint *count_compatible, gboolean *ok )
-{
-	if( positive ){
-		if( match ){
-			*count_compatible += 1;
-		}
-	} else if( match ){
-		*ok = FALSE;
-	}
-}
-
 static gboolean
 is_valid_basenames( const NAIContext *object )
 {
