@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2005 The GNOME Foundation
  * Copyright (C) 2006, 2007, 2008 Frederic Ruaudel and others (see AUTHORS)
- * Copyright (C) 2009 Pierre Wieser and others (see AUTHORS)
+ * Copyright (C) 2009, 2010 Pierre Wieser and others (see AUTHORS)
  *
  * This Program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -35,9 +35,10 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
-#include <runtime/na-iabout.h>
-#include <runtime/na-ipivot-consumer.h>
-#include <runtime/na-object-action-fn.h>
+#include <api/na-core-utils.h>
+
+#include <core/na-iabout.h>
+#include <core/na-ipivot-consumer.h>
 
 #include "nact-application.h"
 #include "nact-main-window.h"
@@ -51,17 +52,28 @@ struct NactApplicationClassPrivate {
 /* private instance data
  */
 struct NactApplicationPrivate {
-	gboolean  dispose_has_run;
-	NAPivot  *pivot;
+	gboolean   dispose_has_run;
+	NAUpdater *updater;
 };
 
 /* private instance properties
  */
 enum {
-	NACT_APPLICATION_PROP_PIVOT_ID = 1
+	NACT_APPLICATION_PROP_UPDATER_ID = 1
 };
 
-#define NACT_APPLICATION_PROP_PIVOT		"nact-application-pivot"
+#define NACT_APPLICATION_PROP_UPDATER	"nact-application-prop-updater"
+
+static gboolean     st_non_unique_opt = FALSE;
+static gboolean     st_version_opt    = FALSE;
+
+static GOptionEntry st_entries[] = {
+	{ "non-unique", 'n', 0, G_OPTION_ARG_NONE, &st_non_unique_opt,
+			N_( "Set it to run multiple instances of the program [unique]" ), NULL },
+	{ "version"   , 'v', 0, G_OPTION_ARG_NONE, &st_version_opt,
+			N_( "Output the version number, and exit gracefully [no]" ), NULL },
+	{ NULL }
+};
 
 static BaseApplicationClass *st_parent_class = NULL;
 
@@ -73,6 +85,7 @@ static void     instance_set_property( GObject *object, guint property_id, const
 static void     instance_dispose( GObject *application );
 static void     instance_finalize( GObject *application );
 
+static gboolean appli_manage_options( BaseApplication *application );
 static gboolean appli_initialize_unique_app( BaseApplication *application );
 static gboolean appli_initialize_application( BaseApplication *application );
 static gchar   *appli_get_application_name( BaseApplication *application );
@@ -137,15 +150,16 @@ class_init( NactApplicationClass *klass )
 	object_class->set_property = instance_set_property;
 
 	spec = g_param_spec_pointer(
-			NACT_APPLICATION_PROP_PIVOT,
-			NACT_APPLICATION_PROP_PIVOT,
-			"NAPivot object pointer",
+			NACT_APPLICATION_PROP_UPDATER,
+			NACT_APPLICATION_PROP_UPDATER,
+			"NAUpdater object pointer",
 			G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE );
-	g_object_class_install_property( object_class, NACT_APPLICATION_PROP_PIVOT_ID, spec );
+	g_object_class_install_property( object_class, NACT_APPLICATION_PROP_UPDATER_ID, spec );
 
 	klass->private = g_new0( NactApplicationClassPrivate, 1 );
 
 	appli_class = BASE_APPLICATION_CLASS( klass );
+	appli_class->manage_options = appli_manage_options;
 	appli_class->initialize_unique_app = appli_initialize_unique_app;
 	appli_class->initialize_application = appli_initialize_application;
 	appli_class->get_application_name = appli_get_application_name;
@@ -156,14 +170,15 @@ class_init( NactApplicationClass *klass )
 }
 
 static void
-instance_init( GTypeInstance *instance, gpointer klass )
+instance_init( GTypeInstance *application, gpointer klass )
 {
 	static const gchar *thisfn = "nact_application_instance_init";
 	NactApplication *self;
 
-	g_debug( "%s: instance=%p, klass=%p", thisfn, ( void * ) instance, ( void * ) klass );
-	g_assert( NACT_IS_APPLICATION( instance ));
-	self = NACT_APPLICATION( instance );
+	g_debug( "%s: application=%p (%s), klass=%p",
+			thisfn, ( void * ) application, G_OBJECT_TYPE_NAME( application ), ( void * ) klass );
+	g_assert( NACT_IS_APPLICATION( application ));
+	self = NACT_APPLICATION( application );
 
 	self->private = g_new0( NactApplicationPrivate, 1 );
 
@@ -181,8 +196,8 @@ instance_get_property( GObject *object, guint property_id, GValue *value, GParam
 	if( !self->private->dispose_has_run ){
 
 		switch( property_id ){
-			case NACT_APPLICATION_PROP_PIVOT_ID:
-				g_value_set_pointer( value, self->private->pivot );
+			case NACT_APPLICATION_PROP_UPDATER_ID:
+				g_value_set_pointer( value, self->private->updater );
 				break;
 
 			default:
@@ -203,8 +218,8 @@ instance_set_property( GObject *object, guint property_id, const GValue *value,
 	if( !self->private->dispose_has_run ){
 
 		switch( property_id ){
-			case NACT_APPLICATION_PROP_PIVOT_ID:
-				self->private->pivot = g_value_get_pointer( value );
+			case NACT_APPLICATION_PROP_UPDATER_ID:
+				self->private->updater = g_value_get_pointer( value );
 				break;
 
 			default:
@@ -220,22 +235,22 @@ instance_dispose( GObject *application )
 	static const gchar *thisfn = "nact_application_instance_dispose";
 	NactApplication *self;
 
-	g_debug( "%s: application=%p", thisfn, ( void * ) application );
+	g_debug( "%s: application=%p (%s)", thisfn, ( void * ) application, G_OBJECT_TYPE_NAME( application ));
 	g_return_if_fail( NACT_IS_APPLICATION( application ));
 	self = NACT_APPLICATION( application );
 
 	if( !self->private->dispose_has_run ){
 
-		if( self->private->pivot ){
-			g_object_unref( self->private->pivot );
+		self->private->dispose_has_run = TRUE;
+
+		if( self->private->updater ){
+			g_object_unref( self->private->updater );
 		}
 
 		/* chain up to the parent class */
 		if( G_OBJECT_CLASS( st_parent_class )->dispose ){
 			G_OBJECT_CLASS( st_parent_class )->dispose( application );
 		}
-
-		self->private->dispose_has_run = TRUE;
 	}
 }
 
@@ -272,30 +287,54 @@ nact_application_new_with_args( int argc, char **argv )
 					NACT_APPLICATION_TYPE,
 					BASE_APPLICATION_PROP_ARGC, argc,
 					BASE_APPLICATION_PROP_ARGV, argv,
+					BASE_APPLICATION_PROP_OPTIONS, st_entries,
 					NULL )
 	);
 }
 
 /**
- * Returns a pointer on the NAPivot object.
- *
+ * nact_application_get_updater:
  * @application: this NactApplication object.
  *
- * The returned pointer is owned by the NactApplication object.
- * It should not be freed not unref by the caller.
+ * Returns a pointer on the #NAUpdater object.
+ *
+ * The returned pointer is owned by the #NactApplication object.
+ * It should not be g_free() not g_object_unref() by the caller.
  */
-NAPivot *
-nact_application_get_pivot( NactApplication *application )
+NAUpdater *
+nact_application_get_updater( NactApplication *application )
 {
-	NAPivot *pivot = NULL;
+	NAUpdater *updater = NULL;
 
 	g_return_val_if_fail( NACT_IS_APPLICATION( application ), NULL );
 
 	if( !application->private->dispose_has_run ){
-		pivot = application->private->pivot;
+
+		updater = application->private->updater;
 	}
 
-	return( pivot );
+	return( updater );
+}
+
+/*
+ * overriden to manage command-line options
+ */
+static gboolean
+appli_manage_options( BaseApplication *application )
+{
+	gboolean ok;
+
+	/* call parent class */
+	ok = BASE_APPLICATION_CLASS( st_parent_class )->manage_options( application );
+
+	if( ok ){
+		if( st_version_opt ){
+			na_core_utils_print_version();
+			ok = FALSE;
+		}
+	}
+
+	return( ok );
 }
 
 /*
@@ -350,14 +389,12 @@ appli_initialize_application( BaseApplication *application )
 {
 	static const gchar *thisfn = "nact_application_appli_initialize_application";
 	gboolean ok;
-	NAObjectAction *fake;
 
 	g_debug( "%s: application=%p", thisfn, ( void * ) application );
 
-	fake = na_object_action_new();
-	g_object_unref( fake );
-
-	NACT_APPLICATION( application )->private->pivot = na_pivot_new( NULL );
+	NACT_APPLICATION( application )->private->updater = na_updater_new();
+	na_pivot_set_loadable( NA_PIVOT( NACT_APPLICATION( application )->private->updater ), PIVOT_LOAD_ALL );
+	na_pivot_load_items( NA_PIVOT( NACT_APPLICATION( application )->private->updater ));
 
 	/* call parent class */
 	ok = BASE_APPLICATION_CLASS( st_parent_class )->initialize_application( application );
@@ -393,6 +430,10 @@ appli_get_unique_app_name( BaseApplication *application )
 
 	g_debug( "%s: application=%p", thisfn, ( void * ) application );
 
+	if( st_non_unique_opt ){
+		return( g_strdup( "" ));
+	}
+
 	return( g_strdup( "org.nautilus-actions.ConfigurationTool" ));
 }
 
@@ -417,7 +458,7 @@ appli_get_main_window( BaseApplication *application )
 	window = BASE_WINDOW( nact_main_window_new( application ));
 
 	na_pivot_register_consumer(
-			nact_application_get_pivot( NACT_APPLICATION( application )),
+			NA_PIVOT( nact_application_get_updater( NACT_APPLICATION( application ))),
 			NA_IPIVOT_CONSUMER( window ));
 
 	return( G_OBJECT( window ));
