@@ -31,6 +31,8 @@
 #ifndef __NACT_MAIN_TAB_H__
 #define __NACT_MAIN_TAB_H__
 
+#include <api/na-icontext.h>
+
 #include "nact-main-window.h"
 
 /**
@@ -43,16 +45,14 @@
 
 /* properties set against the GObject instance
  */
-#define TAB_UPDATABLE_PROP_EDITED_ACTION				"nact-tab-updatable-edited-action"
-#define TAB_UPDATABLE_PROP_EDITED_PROFILE				"nact-tab-updatable-edited-profile"
-#define TAB_UPDATABLE_PROP_SELECTED_ROW					"nact-tab-updatable-selected-row"
+#define TAB_UPDATABLE_PROP_SELECTED_ITEM				"nact-tab-updatable-selected-item"
+#define TAB_UPDATABLE_PROP_SELECTED_PROFILE				"nact-tab-updatable-selected-profile"
 #define TAB_UPDATABLE_PROP_EDITABLE						"nact-tab-updatable-editable"
 #define TAB_UPDATABLE_PROP_REASON						"nact-tab-updatable-reason"
 
 /* signals
  */
 #define TAB_UPDATABLE_SIGNAL_ITEM_UPDATED				"nact-tab-updatable-item-updated"
-#define TAB_UPDATABLE_SIGNAL_ENABLE_TAB					"nact-tab-updatable-enable-tab"
 #define TAB_UPDATABLE_SIGNAL_PROVIDER_CHANGED			"nact-tab-updatable-provider-changed"
 
 /* notebook tabs
@@ -61,13 +61,20 @@
 enum {
 	TAB_ACTION = 0,
 	TAB_COMMAND,
+	TAB_EXECUTION,
+	TAB_BASENAMES,
+	TAB_MIMETYPES,
 	TAB_FOLDERS,
-	TAB_CONDITIONS,
-	TAB_ADVANCED
+	TAB_SCHEMES,
+	TAB_CAPABILITIES,
+	TAB_ENVIRONMENT,
+	TAB_PROPERTIES
 };
 
-void     nact_main_tab_enable_page( NactMainWindow *window, gint page, gboolean enabled );
-gboolean nact_main_tab_is_page_enabled( NactMainWindow *window, gint page );
+void        nact_main_tab_enable_page    ( NactMainWindow *window, gint page, gboolean enabled );
+gboolean    nact_main_tab_is_page_enabled( NactMainWindow *window, gint page );
+
+NAIContext *nact_main_tab_get_context    ( NactMainWindow *window, gboolean *editable );
 
 G_END_DECLS
 
